#ifndef _MARKOV_HPP_
#define _MARKOV_HPP_

#include "matrice.hpp"
#include <iostream>
#include <random>


// Une fonction renvoyant une nombre aleatoire de loi uniforme sur ]0,1[ 
double unif()
{
  static std::mt19937 gen(12345);
  return (gen() + 0.5) / (gen.max() - gen.min());
}


template <int n>
class markov
{
private:
  matrice<n> P;
  int position;
public:
  // Le constructeur prescrivant la position initiale
  markov(int k=0) : position(k), P() {};
  
  void operator= (int k);

  // Une fonction renvoyant le noyau de transition en lecture seule
  matrice<n> noyau(void) const
  {return P;}
  // Une fonction renvoyant la position en lecture seule
  int pos(void) const
  {return position;}
  
  // Iteration de la chaine (les deux operateurs x++ et ++x auront meme effet)
  void operator++(void);
  void operator++(int);

  // Une fonction modifiant les coefficients de la matrice
  void modif_coeff(int, int, double);
  
};


template <int n>
void markov<n>::operator=(int k)
{
  position = k;
}



template<int n>
void markov<n>::modif_coeff(int i, int j, double p)
{
  if (i>=0 && i<n && j>=0 && j<n && i!=j)
	{
	  double somme = p-P(i,j);
	  for (int k=0; k<n; k++)
		if (k != i)
		  somme += P(i,k);
	  // somme vaut la somme des coefficients hors-diagonaux (si la matrice etait modifiee)
	  // Ce nombre doit etre dans [0,1].
	  
	  if (p>=0 && somme<=1)
		{
		  // Si les conditions sont verifiees, on modifie effectivement la matrice
		  P(i,j) = p;
		  P(i,i) = 1-somme;
		}
	}
}


template <int n>
void markov<n>::operator++(void)
{
  double U = unif(), somme = 0;
  int i=-1; 
  // Simulation d'une variable de loi P(position, .)
  while (somme<U)
	{
	  i++;
	  somme += P(position, i);
	}
  position = i;
}

template <int n>
void markov<n>::operator++(int)
{
  ++(*this);
}

#endif /*_MARKOV_HPP_ */
