/*

  Histogramme de la 1024eme iteration d'une marche
  aleatoire asymetrique sur {0,...,N}.

  Avec les parametres donnes, cet histogramme est
  proche de toutes les lignes de P^1024 car 1024
  iterations sont suffisantes pour atteindre la
  mesure invariante de la chaine.
  
*/


#include "markov.hpp"
#include <iostream>


const int n = 100; // Taille de l'espace d'etats.
const double p = 1./3; // Probabilite d'aller a droite.
const int nb_simus = 10000; // Nombres de simulations dans l'histogramme.


int main(void)
{
  markov<n+1> X(n/2);
  // On remplit la matrice de transition
  for (int i=0; i<=n; i++)
	{
	  X.modif_coeff(i, i+1, p);
	  X.modif_coeff(i+1, i, 1-p);
	}
  X.modif_coeff(0, 0, 1-p);
  X.modif_coeff(n-1, n-1, 1-p);


  // On cree l'histogramme
  double *histo = new double[n+1];
  for (int i=0; i<=n; i++)
	histo[i] = 0;
  

  for (int i=0; i<nb_simus; i++)
	{
	  // On fait partir le processus de n/2, et on calcule 1024 iterations.
	  X = n/2;
	  for (int i=0; i<1024; i++)
		++X;
	  // On remplit l'histogramme.
	  histo[X.pos()] += 1./nb_simus;
	}


  matrice<n+1> P = X.noyau();
  for (int j=0; j<10; j++)
	P *= P;
  // La ligne precedente remplace P par P^2. Appellee 10, fois on obtient P^(2^10) = P^1024.


  // On affiche les premieres et dernieres lignes de P et histo, pour comparaison.
  for (int i=0; i<=n; i++)
	std::cout << P(0,i) << " "  << P(n,i) << " " << histo[i] << std::endl;
  
  return 0;
}
