/*

  Affiche l'histogramme du temps d'atteinte de 0 ou n,
  pour la marche aleatoire sur {0,...,n} absorbee en 0
  et en n.
  
*/


#include "markov.hpp"
#include <iostream>

const int n = 20; // Taille de l'espace d'etats
const int nb_simus = 10000; // Nombre de simulations
const int taille_histo = 150; // Nombre de cases dans l'histogramme

int main(void)
{
  markov<n+1> X(n/2);
  // On remplit la matrice
  for (int i=1; i<n; i++)
	{
	  X.modif_coeff(i, i+1, 0.5);
	  X.modif_coeff(i, i-1, 0.5);
	}
  
  // On cree l'histogramme
  double *histo = new double[taille_histo];
  for (int i=0; i<taille_histo; i++)
	histo[i] = 0;

  
  for (int i=0; i<nb_simus; i++)
	{
	  // On part de n/2
	  X = n/2;
	  int t = 0;
	  // On itere la chaine jusqu'a atteindre 0 ou n, en gardant le nombre d'iterations
	  while (X.pos() != 0 && X.pos() != n)
		{
		  t++;
		  X++;
		}
	  // On remplit l'histogramme
	  // (t est forcement pair, on ne garde donc que les t/2).
	  if (t/2<taille_histo)
		histo[t/2] += 1./nb_simus;
	}

  for (int i=0; i<taille_histo; i++)
	std::cout << 2*i << " " << histo[i] << std::endl;
  
  return 0;
}
